///////////////////////////////////////////////////////////
//  IGroup.cs
//  Implementation of the Interface IGroup
//  Generated by Enterprise Architect
//  Created on:      17-May-2018 16:32:19
//  Original author: simon.mettrick
///////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;



using Org.OMG.TACSIT.DataPayload.EntityPayload;
using Org.OMG.TACSIT.DataInterface;
using Org.OMG.TACSIT.DataPayload.GroupPayload;
namespace Org.OMG.TACSIT.DataInterface {


    public interface IEntityChangePublisher
    {

        event EventHandler<Org.OMG.TACSIT.DataPayload.CallbackData.EntityChangeEventList> EntityChange;

    }

    /// <summary>
	/// This interface accesses to a group.
	/// </summary>
	public interface IGroup {

		/// <summary>
		/// Grouped creation of entities.
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		/// <param name="ents"></param>
		void AddEntities(Org.OMG.TACSIT.DataPayload.EntityPayload.EntityList ents);

		/// <summary>
		/// Add an entity to the group and returns the possibly modified entity with its
		/// URI.
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		/// <param name="ent"></param>
		Org.OMG.TACSIT.DataInterface.IEntity AddEntity(Org.OMG.TACSIT.DataPayload.EntityPayload.EntityPayload ent);

		/// <summary>
		/// Adds an entity relationship (a pair-wise relation between two of the group's
		/// entities) to the group.
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		/// <param name="er"></param>
		void AddEntityRelationship(Org.OMG.TACSIT.DataPayload.GroupPayload.EntityRelationship er);

        IEntityChangePublisher GetChangePublisher();

        IEntityChangePublisher GetChangePublisher(int rate);
        
        /// <summary>
		/// Listen to the entity creation, deletion, and update events in the group at a
		/// given rate.
		///        This method may return the InadequateRate exception (if the
		/// implementation deems this parameter as unsuitable) or the TooManyListeners
		/// exception (if the implementation limit the number of listeners) or the
		/// NotImplemented exception if this capability is not available.
		/// </summary>
		/// <param name="l"></param>
		/// <param name="rate"></param>
		void AddListener(Org.OMG.TACSIT.DataInterface.IEntityChangeListener l, int rate);

		/// <summary>
		/// Listen to the entity creation, deletion, and update events in the group.
		///        This method may return the TooManyListeners exception (if the
		/// implementation limit the number of listeners).
		/// </summary>
		/// <param name="l"></param>
		void AddListener(Org.OMG.TACSIT.DataInterface.IEntityChangeListener l);

		/// <summary>
		/// Delete the group.
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		void Delete();

		/// <summary>
		/// Get the interface of an entity from its URI and return the UnknownURI exception
		/// if the entity does not exist (anymore).
		/// </summary>
		/// <param name="uri"></param>
		Org.OMG.TACSIT.DataInterface.IEntity GetEntity(Uri uri);

		/// <summary>
		/// Returns true if the group may be edited according to the TACSIT system.
		/// </summary>
		bool IsEditable();

		/// <summary>
		/// List the references to the entities of the group after filtering them on their
		/// name (namePattern is a regexp).
		/// </summary>
		/// <param name="namePattern"></param>
		Org.OMG.TACSIT.DataPayload.EntityPayload.EntityList ListEntities(string namePattern);

		/// <summary>
		/// List the references to the entities of the group that match the predicate in
		/// argument.
		/// </summary>
		/// <param name="p"></param>
		Org.OMG.TACSIT.DataPayload.EntityPayload.EntityList QueryEntities(Org.OMG.TACSIT.DataInterface.IEntityQuerier p);

		/// <summary>
		/// Get the payload of the group.
		/// </summary>
		Org.OMG.TACSIT.DataPayload.GroupPayload.GroupPayload Read();

		/// <summary>
		/// Remove a previously set listener and return the UnknownListener exception if
		/// the listener does not exist (any more).
		/// </summary>
		/// <param name="l"></param>
		void RemoveChangeListener(Org.OMG.TACSIT.DataInterface.IEntityChangeListener l);

		/// <summary>
		/// Grouped retrieval of entities.
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		/// <param name="ents"></param>
		void RemoveEntities(Org.OMG.TACSIT.DataPayload.EntityPayload.EntityList ents);

		/// <summary>
		/// Remove an entity in the group and return the UnknownEntity exception if the
		/// entity does not exist (anymore).
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		/// <param name="ent"></param>
		void RemoveEntity(Org.OMG.TACSIT.DataPayload.EntityPayload.EntityRef ent);

		/// <summary>
		/// Removes an entity relationship (a pair-wise relation between two of the group's
		/// entities) to the group.
		///        The implementation may return a NotImplemented exception or the Group
		/// has no such entity relationship the UnknownEntityRelationship exception
		/// </summary>
		/// <param name="er"></param>
		void RemoveEntityRelationship(Org.OMG.TACSIT.DataPayload.GroupPayload.EntityRelationship er);

		/// <summary>
		/// Update the meta data of the group and return the new group.
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		/// <param name="gmd"></param>
		Org.OMG.TACSIT.DataPayload.GroupPayload.GroupPayload Update(Org.OMG.TACSIT.DataPayload.GroupPayload.GroupMetaData gmd);

		/// <summary>
		/// Grouped modification of entities.
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		/// <param name="ents"></param>
		void UpdateEntities(Org.OMG.TACSIT.DataPayload.EntityPayload.EntityList ents);

		/// <summary>
		/// Modify an entity in the group (based on its URI) and returns the possibly
		/// modified entity.
		///        The implementation may return a NotImplemented exception.
		/// </summary>
		/// <param name="ent"></param>
		Org.OMG.TACSIT.DataPayload.EntityPayload.EntityPayload UpdateEntity(Org.OMG.TACSIT.DataPayload.EntityPayload.EntityPayload ent);
	}//end IGroup

}//end namespace DataInterface